@@ -111,6 +111,18 @@ static void	dump_leaf_hash(Leaf leaf, int depth, Out out);
 
 static const char	hex_chars[17] = "0123456789abcdef";
 
+// JSON standard except newlines are no escaped
+static char	newline_friendly_chars[256] = "\
+66666666221622666666666666666666\
+11211111111111111111111111111111\
+11111111111111111111111111112111\
+11111111111111111111111111111111\
+11111111111111111111111111111111\
+11111111111111111111111111111111\
+11111111111111111111111111111111\
+11111111111111111111111111111111";
+
+// JSON standard
 static char	hibit_friendly_chars[256] = "\
 66666666222622666666666666666666\
 11211111111111111111111111111111\
@@ -125,7 +137,7 @@ static char	hibit_friendly_chars[256] = "\
 // bytes per character in the output. That makes this conservative.
 static char	ascii_friendly_chars[256] = "\
 66666666222622666666666666666666\
-11211111111111121111111111111111\
+11211111111111111111111111111111\
 11111111111111111111111111112111\
 11111111111111111111111111111116\
 33333333333333333333333333333333\
@@ -133,6 +145,7 @@ static char	ascii_friendly_chars[256] = "\
 33333333333333333333333333333333\
 33333333333333333333333333333333";
 
+// XSS safe mode
 static char	xss_friendly_chars[256] = "\
 66666666222622666666666666666666\
 11211161111111121111111111116161\
@@ -143,6 +156,16 @@ static char	xss_friendly_chars[256] = "\
 33333333333333333333333333333333\
 33333333333333333333333333333333";
 
+inline static size_t
+newline_friendly_size(const uint8_t *str, size_t len) {
+    size_t	size = 0;
+
+    for (; 0 < len; str++, len--) {
+	size += newline_friendly_chars[*str];
+    }
+    return size - len * (size_t)'0';
+}
+
 inline static size_t
 hibit_friendly_size(const uint8_t *str, size_t len) {
     size_t	size = 0;
@@ -445,7 +468,7 @@ dump_float(VALUE obj, Out out) {
     } else if (d == (double)(long long int)d) {
 	cnt = sprintf(buf, "%.1f", d); // used sprintf due to bug in snprintf
     } else {
-	cnt = sprintf(buf, "%0.15g", d); // used sprintf due to bug in snprintf
+	cnt = sprintf(buf, "%0.16g", d); // used sprintf due to bug in snprintf
     }
     if (out->end - out->cur <= (long)cnt) {
 	grow(out, cnt);
@@ -462,6 +485,10 @@ dump_cstr(const char *str, size_t cnt, int is_sym, int escape1, Out out) {
     char	*cmap;
 
     switch (out->opts->escape_mode) {
+    case NLEsc:
+	cmap = newline_friendly_chars;
+	size = newline_friendly_size((uint8_t*)str, cnt);
+	break;
     case ASCIIEsc:
 	cmap = ascii_friendly_chars;
 	size = ascii_friendly_size((uint8_t*)str, cnt);
@@ -512,6 +539,7 @@ dump_cstr(const char *str, size_t cnt, int is_sym, int escape1, Out out) {
 	    case '2':
 		*out->cur++ = '\\';
 		switch (*str) {
+		case '\\':	*out->cur++ = '\\';	break;
 		case '\b':	*out->cur++ = 'b';	break;
 		case '\t':	*out->cur++ = 't';	break;
 		case '\n':	*out->cur++ = 'n';	break;
@@ -969,15 +997,13 @@ dump_time(VALUE obj, Out out) {
     long		nsec = NUM2LONG(rb_funcall2(obj, oj_tv_usec_id, 0, 0)) * 1000;
 #endif
 #endif
-
+    
     if (0 > sec) {
 	neg = 1;
 	sec = -sec;
 	if (0 < nsec) {
 	    nsec = 1000000000 - nsec;
-#ifndef JRUBY_RUBY
 	    sec--;
-#endif
 	}
     }
     *b-- = '\0';
@@ -1209,7 +1235,16 @@ dump_data_obj(VALUE obj, int depth, Out out) {
 	*out->cur++ = 't';
 	*out->cur++ = '"';
 	*out->cur++ = ':';
-	dump_time(obj, out);
+	switch (out->opts->time_format) {
+	case RubyTime: // Does not output fractional seconds
+	case XmlTime:
+	    dump_xml_time(obj, out);
+	    break;
+	case UnixTime:
+	default:
+	    dump_time(obj, out);
+	    break;
+	}
 	*out->cur++ = '}';
 	*out->cur = '\0';
     } else {
@@ -1236,7 +1271,7 @@ dump_obj_comp(VALUE obj, int depth, Out out) {
 	    rb_raise(rb_eTypeError, "%s.to_hash() did not return a Hash.\n", rb_class2name(rb_obj_class(obj)));
 	}
 	dump_hash(h, Qundef, depth, out->opts->mode, out);
-    } else if (rb_respond_to(obj, oj_as_json_id)) {
+    } else if (Yes == out->opts->to_json && rb_respond_to(obj, oj_as_json_id)) {
 	dump_val(rb_funcall(obj, oj_as_json_id, 0), depth, out);
     } else if (Yes == out->opts->to_json && rb_respond_to(obj, oj_to_json_id)) {
 	volatile VALUE	rs;
@@ -1264,7 +1299,11 @@ dump_obj_comp(VALUE obj, int depth, Out out) {
 	    } else {
 		dump_cstr(rb_string_value_ptr((VALUE*)&rstr), RSTRING_LEN(rstr), 0, 0, out);
 	    }
+#if (defined T_RATIONAL && defined RRATIONAL)
+	} else if (oj_datetime_class == clas || oj_date_class == clas || rb_cRational == clas) {
+#else
 	} else if (oj_datetime_class == clas || oj_date_class == clas) {
+#endif
 	    volatile VALUE	rstr = rb_funcall(obj, oj_to_s_id, 0);
 
 	    dump_cstr(rb_string_value_ptr((VALUE*)&rstr), RSTRING_LEN(rstr), 0, 0, out);
@@ -2312,7 +2351,7 @@ oj_str_writer_pop(StrWriter sw) {
 	*sw->out.cur++ = ']';
 	break;
     }
-    if (0 == sw->depth && 0 < sw->out.indent) {
+    if (0 == sw->depth && 0 <= sw->out.indent) {
 	*sw->out.cur++ = '\n';
     }
 }
