@@ -53,15 +53,15 @@ read_long(const char *str, size_t len) {
 }
 
 static VALUE
-hash_key(ParseInfo pi, const char *key, size_t klen, char k1) {
+calc_hash_key(ParseInfo pi, Val kval, char k1) {
     VALUE	rkey;
 
     if (':' == k1) {
-	rkey = rb_str_new(key + 1, klen - 1);
+	rkey = rb_str_new(kval->key + 1, kval->klen - 1);
 	rkey = oj_encode(rkey);
 	rkey = rb_funcall(rkey, oj_to_sym_id, 0);
     } else {
-	rkey = rb_str_new(key, klen);
+	rkey = rb_str_new(kval->key, kval->klen);
 	rkey = oj_encode(rkey);
 	if (Yes == pi->options.sym_key) {
 	    rkey = rb_str_intern(rkey);
@@ -94,7 +94,10 @@ str_to_value(ParseInfo pi, const char *str, size_t len, const char *orig) {
 }
 
 static int
-hat_cstr(ParseInfo pi, Val parent, const char *key, size_t klen, const char *str, size_t len) {
+hat_cstr(ParseInfo pi, Val parent, Val kval, const char *str, size_t len) {
+    const char	*key = kval->key;
+    int		klen = kval->klen;
+
     if (2 == klen) {
 	switch (key[1]) {
 	case 'o': // object
@@ -129,6 +132,9 @@ hat_cstr(ParseInfo pi, Val parent, const char *key, size_t klen, const char *str
 	case 'c': // class
 	    parent->val = oj_name2class(pi, str, len, Yes == pi->options.auto_define);
 	    break;
+	case 't': // time
+	    parent->val = rb_funcall(oj_time_class, rb_intern("parse"), 1, rb_str_new(str, len));
+	    break;
 	default:
 	    return 0;
 	    break;
@@ -139,9 +145,9 @@ hat_cstr(ParseInfo pi, Val parent, const char *key, size_t klen, const char *str
 }
 
 static int
-hat_num(ParseInfo pi, Val parent, const char *key, size_t klen, NumInfo ni) {
-    if (2 == klen) {
-	switch (key[1]) {
+hat_num(ParseInfo pi, Val parent, Val kval, NumInfo ni) {
+    if (2 == kval->klen) {
+	switch (kval->key[1]) {
 	case 't': // time as a float
 	    {
 		int64_t	nsec = ni->num * 1000000000LL / ni->div;
@@ -158,6 +164,9 @@ hat_num(ParseInfo pi, Val parent, const char *key, size_t klen, NumInfo ni) {
 #else
 		parent->val = rb_time_new(ni->i, (long)(nsec / 1000));
 #endif
+		// TBD if there is an exponent then if not gmt_offset
+		// add gmt_offset and then getgmt()
+		// if not 0 then getlocal(offset)
 	    }
 	    break;
 	case 'i': // circular index
@@ -256,9 +265,11 @@ copy_ivars(VALUE target, VALUE src) {
 }
 
 static void
-set_obj_ivar(Val parent, const char *key, size_t klen, VALUE value) {
-    ID	var_id;
-    ID	*slot;
+set_obj_ivar(Val parent, Val kval, VALUE value) {
+    const char	*key = kval->key;
+    int		klen = kval->klen;
+    ID		var_id;
+    ID		*slot;
 
     if ('~' == *key && Qtrue == rb_obj_is_kind_of(parent->val, rb_eException)) {
 	if (5 == klen && 0 == strncmp("~mesg", key, klen)) {
@@ -280,7 +291,7 @@ set_obj_ivar(Val parent, const char *key, size_t klen, VALUE value) {
     if (0 == (var_id = oj_attr_hash_get(key, klen, &slot))) {
 	char	attr[256];
 
-	if (sizeof(attr) <= klen + 2) {
+	if ((int)sizeof(attr) <= klen + 2) {
 	    char	*buf = ALLOC_N(char, klen + 2);
 
 	    if ('~' == *key) {
@@ -315,39 +326,41 @@ set_obj_ivar(Val parent, const char *key, size_t klen, VALUE value) {
 }
 
 static void
-hash_set_cstr(ParseInfo pi, const char *key, size_t klen, const char *str, size_t len, const char *orig) {
-    Val	parent = stack_peek(&pi->stack);
+hash_set_cstr(ParseInfo pi, Val kval, const char *str, size_t len, const char *orig) {
+    const char	*key = kval->key;
+    int		klen = kval->klen;
+    Val		parent = stack_peek(&pi->stack);
 
  WHICH_TYPE:
     switch (rb_type(parent->val)) {
     case T_NIL:
 	parent->odd_args = 0; // make sure it is 0 in case not odd
-	if ('^' != *key || !hat_cstr(pi, parent, key, klen, str, len)) {
+	if ('^' != *key || !hat_cstr(pi, parent, kval, str, len)) {
 	    parent->val = rb_hash_new();
 	    goto WHICH_TYPE;
 	}
 	break;
     case T_HASH:
-	rb_hash_aset(parent->val, hash_key(pi, key, klen, parent->k1), str_to_value(pi, str, len, orig));
+	rb_hash_aset(parent->val, calc_hash_key(pi, kval, parent->k1), str_to_value(pi, str, len, orig));
 	break;
     case T_STRING:
 	if (4 == klen && 's' == *key && 'e' == key[1] && 'l' == key[2] && 'f' == key[3]) {
 	    rb_funcall(parent->val, oj_replace_id, 1, str_to_value(pi, str, len, orig));
 	} else {
-	    set_obj_ivar(parent, key, klen, str_to_value(pi, str, len, orig));
+	    set_obj_ivar(parent, kval, str_to_value(pi, str, len, orig));
 	}
 	break;
     case T_OBJECT:
-	set_obj_ivar(parent, key, klen, str_to_value(pi, str, len, orig));
+	set_obj_ivar(parent, kval, str_to_value(pi, str, len, orig));
 	break;
     case T_CLASS:
 	if (0 == parent->odd_args) {
 	    oj_set_error_at(pi, oj_parse_error_class, __FILE__, __LINE__, "%s is not an odd class", rb_class2name(rb_obj_class(parent->val)));
 	    return;
-	} else if (0 != oj_odd_set_arg(parent->odd_args, key, klen, str_to_value(pi, str, len, orig))) {
+	} else if (0 != oj_odd_set_arg(parent->odd_args, kval->key, kval->klen, str_to_value(pi, str, len, orig))) {
 	    char	buf[256];
 
-	    if (sizeof(buf) - 1 <= klen) {
+	    if ((int)sizeof(buf) - 1 <= klen) {
 		klen = sizeof(buf) - 2;
 	    }
 	    memcpy(buf, key, klen);
@@ -362,27 +375,29 @@ hash_set_cstr(ParseInfo pi, const char *key, size_t klen, const char *str, size_
 }
 
 static void
-hash_set_num(ParseInfo pi, const char *key, size_t klen, NumInfo ni) {
-    Val	parent = stack_peek(&pi->stack);
+hash_set_num(ParseInfo pi, Val kval, NumInfo ni) {
+    const char	*key = kval->key;
+    int		klen = kval->klen;
+    Val		parent = stack_peek(&pi->stack);
 
  WHICH_TYPE:
     switch (rb_type(parent->val)) {
     case T_NIL:
 	parent->odd_args = 0; // make sure it is 0 in case not odd
-	if ('^' != *key || !hat_num(pi, parent, key, klen, ni)) {
+	if ('^' != *key || !hat_num(pi, parent, kval, ni)) {
 	    parent->val = rb_hash_new();
 	    goto WHICH_TYPE;
 	}
 	break;
     case T_HASH:
-	rb_hash_aset(parent->val, hash_key(pi, key, klen, parent->k1), oj_num_as_value(ni));
+	rb_hash_aset(parent->val, calc_hash_key(pi, kval, parent->k1), oj_num_as_value(ni));
 	break;
     case T_OBJECT:
 	if (2 == klen && '^' == *key && 'i' == key[1] &&
 	    !ni->infinity && !ni->neg && 1 == ni->div && 0 == ni->exp && 0 != pi->circ_array) { // fixnum
 	    oj_circ_array_set(pi->circ_array, parent->val, ni->i);
 	} else {
-	    set_obj_ivar(parent, key, klen, oj_num_as_value(ni));
+	    set_obj_ivar(parent, kval, oj_num_as_value(ni));
 	}
 	break;
     case T_CLASS:
@@ -392,7 +407,7 @@ hash_set_num(ParseInfo pi, const char *key, size_t klen, NumInfo ni) {
 	} else if (0 != oj_odd_set_arg(parent->odd_args, key, klen, oj_num_as_value(ni))) {
 	    char	buf[256];
 
-	    if (sizeof(buf) - 1 <= klen) {
+	    if ((int)sizeof(buf) - 1 <= klen) {
 		klen = sizeof(buf) - 2;
 	    }
 	    memcpy(buf, key, klen);
@@ -407,8 +422,10 @@ hash_set_num(ParseInfo pi, const char *key, size_t klen, NumInfo ni) {
 }
 
 static void
-hash_set_value(ParseInfo pi, const char *key, size_t klen, VALUE value) {
-    Val	parent = stack_peek(&pi->stack);
+hash_set_value(ParseInfo pi, Val kval, VALUE value) {
+    const char	*key = kval->key;
+    int		klen = kval->klen;
+    Val		parent = stack_peek(&pi->stack);
 
  WHICH_TYPE:
     switch (rb_type(parent->val)) {
@@ -420,15 +437,14 @@ hash_set_value(ParseInfo pi, const char *key, size_t klen, VALUE value) {
 	}
 	break;
     case T_HASH:
-
 	if (rb_cHash != rb_obj_class(parent->val)) {
 	    if (4 == klen && 's' == *key && 'e' == key[1] && 'l' == key[2] && 'f' == key[3]) {
 		rb_funcall(parent->val, oj_replace_id, 1, value);
 	    } else {
-		set_obj_ivar(parent, key, klen, value);
+		set_obj_ivar(parent, kval, value);
 	    }
 	} else {
-	    if (3 <= klen && '#' == key[1] && T_ARRAY == rb_type(value)) {
+	    if (3 <= klen && '^' == *key && '#' == key[1] && T_ARRAY == rb_type(value)) {
 		long	len = RARRAY_LEN(value);
 		VALUE	*a = RARRAY_PTR(value);
 	
@@ -438,7 +454,7 @@ hash_set_value(ParseInfo pi, const char *key, size_t klen, VALUE value) {
 		}
 		rb_hash_aset(parent->val, *a, a[1]);
 	    } else {
-		rb_hash_aset(parent->val, hash_key(pi, key, klen, parent->k1), value);
+		rb_hash_aset(parent->val, calc_hash_key(pi, kval, parent->k1), value);
 	    }
 	}
 	break;
@@ -446,12 +462,12 @@ hash_set_value(ParseInfo pi, const char *key, size_t klen, VALUE value) {
 	if (4 == klen && 's' == *key && 'e' == key[1] && 'l' == key[2] && 'f' == key[3]) {
 	    rb_funcall(parent->val, oj_replace_id, 1, value);
 	} else {
-	    set_obj_ivar(parent, key, klen, value);
+	    set_obj_ivar(parent, kval, value);
 	}
 	break;
     case T_STRING: // for subclassed strings
     case T_OBJECT:
-	set_obj_ivar(parent, key, klen, value);
+	set_obj_ivar(parent, kval, value);
 	break;
     case T_CLASS:
 	if (0 == parent->odd_args) {
@@ -460,7 +476,7 @@ hash_set_value(ParseInfo pi, const char *key, size_t klen, VALUE value) {
 	} else if (0 !=	oj_odd_set_arg(parent->odd_args, key, klen, value)) {
 	    char	buf[256];
 
-	    if (sizeof(buf) - 1 <= klen) {
+	    if ((int)sizeof(buf) - 1 <= klen) {
 		klen = sizeof(buf) - 2;
 	    }
 	    memcpy(buf, key, klen);
@@ -517,11 +533,21 @@ array_append_cstr(ParseInfo pi, const char *str, size_t len, const char *orig) {
     rb_ary_push(stack_peek(&pi->stack)->val, str_to_value(pi, str, len, orig));
 }
 
+static void
+array_append_num(ParseInfo pi, NumInfo ni) {
+    rb_ary_push(stack_peek(&pi->stack)->val, oj_num_as_value(ni));
+}
+
 static void
 add_cstr(ParseInfo pi, const char *str, size_t len, const char *orig) {
     pi->stack.head->val = str_to_value(pi, str, len, orig);
 }
 
+static void
+add_num(ParseInfo pi, NumInfo ni) {
+    pi->stack.head->val = oj_num_as_value(ni);
+}
+
 void
 oj_set_object_callbacks(ParseInfo pi) {
     oj_set_strict_callbacks(pi);
@@ -531,7 +557,9 @@ oj_set_object_callbacks(ParseInfo pi) {
     pi->hash_set_num = hash_set_num;
     pi->hash_set_value = hash_set_value;
     pi->add_cstr = add_cstr;
+    pi->add_num = add_num;
     pi->array_append_cstr = array_append_cstr;
+    pi->array_append_num = array_append_num;
 }
 
 VALUE
@@ -539,6 +567,7 @@ oj_object_parse(int argc, VALUE *argv, VALUE self) {
     struct _ParseInfo	pi;
 
     pi.options = oj_default_options;
+    pi.handler = Qnil;
     oj_set_object_callbacks(&pi);
 
     if (T_STRING == rb_type(*argv)) {
@@ -553,6 +582,7 @@ oj_object_parse_cstr(int argc, VALUE *argv, char *json, size_t len) {
     struct _ParseInfo	pi;
 
     pi.options = oj_default_options;
+    pi.handler = Qnil;
     oj_set_strict_callbacks(&pi);
     pi.end_hash = end_hash;
     pi.start_hash = start_hash;
@@ -560,7 +590,9 @@ oj_object_parse_cstr(int argc, VALUE *argv, char *json, size_t len) {
     pi.hash_set_num = hash_set_num;
     pi.hash_set_value = hash_set_value;
     pi.add_cstr = add_cstr;
+    pi.add_num = add_num;
     pi.array_append_cstr = array_append_cstr;
+    pi.array_append_num = array_append_num;
 
     return oj_pi_parse(argc, argv, &pi, json, len, 1);
 }
