@@ -58,8 +58,11 @@ ID	oj_array_end_id;
 ID	oj_array_start_id;
 ID	oj_as_json_id;
 ID	oj_error_id;
+ID	oj_file_id;
 ID	oj_fileno_id;
+ID	oj_ftype_id;
 ID	oj_hash_end_id;
+ID	oj_hash_key_id;
 ID	oj_hash_set_id;
 ID	oj_hash_start_id;
 ID	oj_iconv_id;
@@ -67,9 +70,11 @@ ID	oj_instance_variables_id;
 ID	oj_json_create_id;
 ID	oj_length_id;
 ID	oj_new_id;
+ID	oj_pos_id;
 ID	oj_read_id;
 ID	oj_readpartial_id;
 ID	oj_replace_id;
+ID	oj_stat_id;
 ID	oj_string_id;
 ID	oj_to_hash_id;
 ID	oj_to_json_id;
@@ -113,9 +118,11 @@ static VALUE	float_sym;
 static VALUE	indent_sym;
 static VALUE	json_sym;
 static VALUE	mode_sym;
+static VALUE	newline_sym;
 static VALUE	nilnil_sym;
 static VALUE	null_sym;
 static VALUE	object_sym;
+static VALUE	quirks_mode_sym;
 static VALUE	ruby_sym;
 static VALUE	sec_prec_sym;
 static VALUE	strict_sym;
@@ -165,6 +172,7 @@ struct _Options	oj_default_options = {
     10,			// create_id_len
     9,			// sec_prec
     Yes,		// allow_gc
+    Yes,		// quirks_mode
     0,			// dump_opts
 };
 
@@ -173,11 +181,11 @@ static VALUE	define_mimic_json(int argc, VALUE *argv, VALUE self);
 /* call-seq: default_options() => Hash
  *
  * Returns the default load and dump options as a Hash. The options are
- * - indent: [Fixnum] number of spaces to indent each element in an JSON document
+ * - indent: [Fixnum] number of spaces to indent each element in an JSON document, zero is no newline between JSON elements, negative indicates no newline between top level JSON elements in a stream
  * - circular: [true|false|nil] support circular references while dumping
  * - auto_define: [true|false|nil] automatically define classes if they do not exist
  * - symbol_keys: [true|false|nil] use symbols instead of strings for hash keys
- * - escape_mode: [:json|:xss_safe|:ascii|nil] use symbols instead of strings for hash keys
+ * - escape_mode: [:newline|:json|:xss_safe|:ascii|nil] determines the characters to escape
  * - class_cache: [true|false|nil] cache classes for faster parsing (if dynamically modifying classes or reloading classes then don't use this)
  * - mode: [:object|:strict|:compat|:null] load and dump modes to use for JSON
  * - time_format: [:unix|:xmlschema|:ruby] time format when dumping in :compat mode
@@ -188,6 +196,7 @@ static VALUE	define_mimic_json(int argc, VALUE *argv, VALUE self);
  * - use_to_json: [true|false|nil] call to_json() methods on dump, default is false
  * - nilnil: [true|false|nil] if true a nil input to load will return nil and not raise an Exception
  * - allow_gc: [true|false|nil] allow or prohibit GC during parsing, default is true (allow)
+ * - quirks_mode: [true,|false|nil] Allow single JSON values instead of documents, default is true (allow)
  * @return [Hash] all current option settings.
  */
 static VALUE
@@ -204,6 +213,7 @@ get_def_opts(VALUE self) {
     rb_hash_aset(opts, use_to_json_sym, (Yes == oj_default_options.to_json) ? Qtrue : ((No == oj_default_options.to_json) ? Qfalse : Qnil));
     rb_hash_aset(opts, nilnil_sym, (Yes == oj_default_options.nilnil) ? Qtrue : ((No == oj_default_options.nilnil) ? Qfalse : Qnil));
     rb_hash_aset(opts, allow_gc_sym, (Yes == oj_default_options.allow_gc) ? Qtrue : ((No == oj_default_options.allow_gc) ? Qfalse : Qnil));
+    rb_hash_aset(opts, quirks_mode_sym, (Yes == oj_default_options.quirks_mode) ? Qtrue : ((No == oj_default_options.quirks_mode) ? Qfalse : Qnil));
     switch (oj_default_options.mode) {
     case StrictMode:	rb_hash_aset(opts, mode_sym, strict_sym);	break;
     case CompatMode:	rb_hash_aset(opts, mode_sym, compat_sym);	break;
@@ -212,6 +222,7 @@ get_def_opts(VALUE self) {
     default:		rb_hash_aset(opts, mode_sym, object_sym);	break;
     }
     switch (oj_default_options.escape_mode) {
+    case NLEsc:		rb_hash_aset(opts, escape_mode_sym, newline_sym);	break;
     case JSONEsc:	rb_hash_aset(opts, escape_mode_sym, json_sym);		break;
     case XSSEsc:	rb_hash_aset(opts, escape_mode_sym, xss_safe_sym);	break;
     case ASCIIEsc:	rb_hash_aset(opts, escape_mode_sym, ascii_sym);		break;
@@ -243,8 +254,9 @@ get_def_opts(VALUE self) {
  * @param [true|false|nil] :auto_define automatically define classes if they do not exist
  * @param [true|false|nil] :symbol_keys convert hash keys to symbols
  * @param [true|false|nil] :class_cache cache classes for faster parsing
- * @param [:json|:xss_safe|:ascii|nil] :escape mode encodes all high-bit characters as
+ * @param [:newline|:json|:xss_safe|:ascii|nil] :escape mode encodes all high-bit characters as
  *        escaped sequences if :ascii, :json is standand UTF-8 JSON encoding,
+ *        :newline is the same as :json but newlines are not escaped,
  *        and :xss_safe escapes &, <, and >, and some others.
  * @param [true|false|nil] :bigdecimal_as_decimal dump BigDecimal as a decimal number or as a String
  * @param [:bigdecimal|:float|:auto|nil] :bigdecimal_load load decimals as BigDecimal instead of as a Float. :auto pick the most precise for the number of digits.
@@ -265,6 +277,7 @@ get_def_opts(VALUE self) {
  * @param [true|false|nil] :use_to_json call to_json() methods on dump, default is false
  * @param [true|false|nil] :nilnil if true a nil input to load will return nil and not raise an Exception
  * @param [true|false|nil] :allow_gc allow or prohibit GC during parsing, default is true (allow)
+ * @param [true|false|nil] :quirks_mode allow single JSON values instead of documents, default is true (allow)
  * @return [nil]
  */
 static VALUE
@@ -278,6 +291,7 @@ set_def_opts(VALUE self, VALUE opts) {
 	{ use_to_json_sym, &oj_default_options.to_json },
 	{ nilnil_sym, &oj_default_options.nilnil },
 	{ allow_gc_sym, &oj_default_options.allow_gc },
+	{ quirks_mode_sym, &oj_default_options.quirks_mode },
 	{ Qnil, 0 }
     };
     YesNoOpt	o;
@@ -334,6 +348,8 @@ set_def_opts(VALUE self, VALUE opts) {
     v = rb_hash_lookup(opts, escape_mode_sym);
     if (Qnil == v) {
 	// ignore
+    } else if (newline_sym == v) {
+	oj_default_options.escape_mode = NLEsc;
     } else if (json_sym == v) {
 	oj_default_options.escape_mode = JSONEsc;
     } else if (xss_safe_sym == v) {
@@ -341,7 +357,7 @@ set_def_opts(VALUE self, VALUE opts) {
     } else if (ascii_sym == v) {
 	oj_default_options.escape_mode = ASCIIEsc;
     } else {
-	rb_raise(rb_eArgError, ":encoding must be :json, :rails, or :ascii.");
+	rb_raise(rb_eArgError, ":encoding must be :json, :xss_safe, or :ascii.");
     }
 
     v = rb_hash_lookup(opts, bigdecimal_load_sym);
@@ -410,6 +426,7 @@ oj_parse_options(VALUE ropts, Options copts) {
 	{ use_to_json_sym, &copts->to_json },
 	{ nilnil_sym, &copts->nilnil },
 	{ allow_gc_sym, &copts->allow_gc },
+	{ quirks_mode_sym, &copts->quirks_mode },
 	{ Qnil, 0 }
     };
     YesNoOpt	o;
@@ -463,14 +480,16 @@ oj_parse_options(VALUE ropts, Options copts) {
 	}
 
 	if (Qnil != (v = rb_hash_lookup(ropts, escape_mode_sym))) {
-	    if (json_sym == v) {
+	    if (newline_sym == v) {
+		copts->escape_mode = NLEsc;
+	    } else if (json_sym == v) {
 		copts->escape_mode = JSONEsc;
 	    } else if (xss_safe_sym == v) {
 		copts->escape_mode = XSSEsc;
 	    } else if (ascii_sym == v) {
 		copts->escape_mode = ASCIIEsc;
 	    } else {
-		rb_raise(rb_eArgError, ":encoding must be :json, :rails, or :ascii.");
+		rb_raise(rb_eArgError, ":encoding must be :newline, :json, :xss_safe, or :ascii.");
 	    }
 	}
 
@@ -700,6 +719,7 @@ load_file(int argc, VALUE *argv, VALUE self) {
     }
     Check_Type(*argv, T_STRING);
     pi.options = oj_default_options;
+    pi.handler = Qnil;
     if (2 <= argc) {
 	VALUE	ropts = argv[1];
 	VALUE	v;
@@ -1225,9 +1245,10 @@ stream_writer_new(int argc, VALUE *argv, VALUE self) {
     }
     sw = ALLOC(struct _StreamWriter);
     str_writer_init(&sw->sw);
-    if (1 == argc) {
-	oj_parse_options(argv[0], &sw->sw.opts);
+    if (2 == argc) {
+	oj_parse_options(argv[1], &sw->sw.opts);
     }
+    sw->sw.out.indent = sw->sw.opts.indent;
     sw->stream = stream;
     sw->type = type;
     sw->fd = fd;
@@ -1621,6 +1642,8 @@ mimic_parse(int argc, VALUE *argv, VALUE self) {
     oj_set_compat_callbacks(&pi);
     pi.options = oj_default_options;
     pi.options.auto_define = No;
+    pi.options.quirks_mode = No;
+
     if (2 <= argc) {
 	VALUE	ropts = argv[1];
 	VALUE	v;
@@ -1631,6 +1654,11 @@ mimic_parse(int argc, VALUE *argv, VALUE self) {
 	if (Qnil != (v = rb_hash_lookup(ropts, symbolize_names_sym))) {
 	    pi.options.sym_key = (Qtrue == v) ? Yes : No;
 	}
+
+	if (Qnil != (v = rb_hash_lookup(ropts, quirks_mode_sym))) {
+	    pi.options.quirks_mode = (Qtrue == v) ? Yes : No;
+	}
+
 	if (Qnil != (v = rb_hash_lookup(ropts, create_additions_sym))) {
 	    if (Qfalse == v) {
 		oj_set_strict_callbacks(&pi);
@@ -1680,6 +1708,50 @@ mimic_create_id(VALUE self, VALUE id) {
     return id;
 }
 
+static struct _Options	mimic_object_to_json_options = {
+    0,			// indent
+    No,			// circular
+    No,			// auto_define
+    No,			// sym_key
+    JSONEsc,		// escape_mode
+    CompatMode,		// mode
+    No,			// class_cache
+    RubyTime,		// time_format
+    Yes,		// bigdec_as_num
+    AutoDec,		// bigdec_load
+    No,			// to_json
+    Yes,		// nilnil
+    json_class,		// create_id
+    10,			// create_id_len
+    9,			// sec_prec
+    Yes,		// allow_gc
+    Yes,		// quirks_mode
+    0,			// dump_opts
+};
+
+static VALUE
+mimic_object_to_json(int argc, VALUE *argv, VALUE self) {
+    char		buf[4096];
+    struct _Out		out;
+    VALUE		rstr;
+    
+    out.buf = buf;
+    out.end = buf + sizeof(buf) - 10;
+    out.allocated = 0;
+    //oj_dump_obj_to_json(self, &mimic_object_to_json_options, &out);
+    oj_dump_obj_to_json(self, &oj_default_options, &out);
+    if (0 == out.buf) {
+	rb_raise(rb_eNoMemError, "Not enough memory.");
+    }
+    rstr = rb_str_new2(out.buf);
+    rstr = oj_encode(rstr);
+    if (out.allocated) {
+	xfree(out.buf);
+    }
+    return rstr;
+}
+
+
 /* Document-method: mimic_JSON
  *    call-seq: mimic_JSON() => Module
  *
@@ -1752,6 +1824,9 @@ define_mimic_json(int argc, VALUE *argv, VALUE self) {
 
     rb_define_module_function(mimic, "parse", mimic_parse, -1);
     rb_define_module_function(mimic, "parse!", mimic_parse, -1);
+
+    rb_define_method(rb_cObject, "to_json", mimic_object_to_json, -1);
+
     rb_gv_set("$VERBOSE", dummy);
 
     array_nl_sym = ID2SYM(rb_intern("array_nl"));			rb_gc_register_address(&array_nl_sym);
@@ -1761,12 +1836,16 @@ define_mimic_json(int argc, VALUE *argv, VALUE self) {
     space_sym = ID2SYM(rb_intern("space"));				rb_gc_register_address(&space_sym);
     symbolize_names_sym = ID2SYM(rb_intern("symbolize_names"));		rb_gc_register_address(&symbolize_names_sym);
 
-    if (!rb_const_defined_at(mimic, rb_intern("ParserError"))) {
-	rb_define_const(mimic, "ParserError", oj_parse_error_class);
+    if (rb_const_defined_at(mimic, rb_intern("ParserError"))) {
+	rb_funcall(mimic, rb_intern("remove_const"), 1, ID2SYM(rb_intern("ParserError")));
     }
-    oj_default_options.mode = CompatMode;
-    oj_default_options.escape_mode = ASCIIEsc;
-    oj_default_options.nilnil = Yes;
+    rb_define_const(mimic, "ParserError", oj_parse_error_class);
+
+    if (!rb_const_defined_at(mimic, rb_intern("State"))) {
+        rb_define_class_under(mimic, "State", rb_cObject);
+    }
+
+    oj_default_options = mimic_object_to_json_options;
 
     return mimic;
 }
@@ -1800,6 +1879,7 @@ iconv_rescue(VALUE x) {
 
 static VALUE
 protect_require(VALUE x) {
+    rb_require("time");
     rb_require("bigdecimal");
     return Qnil;
 }
@@ -1876,8 +1956,11 @@ void Init_oj() {
     oj_array_start_id = rb_intern("array_start");
     oj_as_json_id = rb_intern("as_json");
     oj_error_id = rb_intern("error");
+    oj_file_id = rb_intern("file?");
     oj_fileno_id = rb_intern("fileno");
+    oj_ftype_id = rb_intern("ftype");
     oj_hash_end_id = rb_intern("hash_end");
+    oj_hash_key_id = rb_intern("hash_key");
     oj_hash_set_id = rb_intern("hash_set");
     oj_hash_start_id = rb_intern("hash_start");
     oj_iconv_id = rb_intern("iconv");
@@ -1885,9 +1968,11 @@ void Init_oj() {
     oj_json_create_id = rb_intern("json_create");
     oj_length_id = rb_intern("length");
     oj_new_id = rb_intern("new");
+    oj_pos_id = rb_intern("pos");
     oj_read_id = rb_intern("read");
     oj_readpartial_id = rb_intern("readpartial");
     oj_replace_id = rb_intern("replace");
+    oj_stat_id = rb_intern("stat");
     oj_string_id = rb_intern("string");
     oj_to_hash_id = rb_intern("to_hash");
     oj_to_json_id = rb_intern("to_json");
@@ -1924,7 +2009,6 @@ void Init_oj() {
     bigdecimal_load_sym = ID2SYM(rb_intern("bigdecimal_load"));rb_gc_register_address(&bigdecimal_load_sym);
     bigdecimal_sym = ID2SYM(rb_intern("bigdecimal"));	rb_gc_register_address(&bigdecimal_sym);
     circular_sym = ID2SYM(rb_intern("circular"));	rb_gc_register_address(&circular_sym);
-    nilnil_sym = ID2SYM(rb_intern("nilnil"));		rb_gc_register_address(&nilnil_sym);
     class_cache_sym = ID2SYM(rb_intern("class_cache"));	rb_gc_register_address(&class_cache_sym);
     compat_sym = ID2SYM(rb_intern("compat"));		rb_gc_register_address(&compat_sym);
     create_id_sym = ID2SYM(rb_intern("create_id"));	rb_gc_register_address(&create_id_sym);
@@ -1933,8 +2017,11 @@ void Init_oj() {
     indent_sym = ID2SYM(rb_intern("indent"));		rb_gc_register_address(&indent_sym);
     json_sym = ID2SYM(rb_intern("json"));		rb_gc_register_address(&json_sym);
     mode_sym = ID2SYM(rb_intern("mode"));		rb_gc_register_address(&mode_sym);
+    newline_sym = ID2SYM(rb_intern("newline"));		rb_gc_register_address(&newline_sym);
+    nilnil_sym = ID2SYM(rb_intern("nilnil"));		rb_gc_register_address(&nilnil_sym);
     null_sym = ID2SYM(rb_intern("null"));		rb_gc_register_address(&null_sym);
     object_sym = ID2SYM(rb_intern("object"));		rb_gc_register_address(&object_sym);
+    quirks_mode_sym = ID2SYM(rb_intern("quirks_mode"));	rb_gc_register_address(&quirks_mode_sym);
     ruby_sym = ID2SYM(rb_intern("ruby"));		rb_gc_register_address(&ruby_sym);
     sec_prec_sym = ID2SYM(rb_intern("second_precision"));rb_gc_register_address(&sec_prec_sym);
     strict_sym = ID2SYM(rb_intern("strict"));		rb_gc_register_address(&strict_sym);
