@@ -87,6 +87,7 @@ typedef enum {
 } TimeFormat;
 
 typedef enum {
+    NLEsc	= 'n',
     JSONEsc	= 'j',
     XSSEsc	= 'x',
     ASCIIEsc	= 'a'
@@ -141,6 +142,7 @@ typedef struct _Options {
     size_t	create_id_len;	// length of create_id
     int		sec_prec;	// second precision when dumping time
     char	allow_gc;	// allow GC during parse
+    char	quirks_mode;	// allow single JSON values instead of documents
     DumpOpts	dump_opts;
 } *Options;
 
@@ -253,8 +255,11 @@ extern ID	oj_array_end_id;
 extern ID	oj_array_start_id;
 extern ID	oj_as_json_id;
 extern ID	oj_error_id;
+extern ID	oj_file_id;
 extern ID	oj_fileno_id;
+extern ID	oj_ftype_id;
 extern ID	oj_hash_end_id;
+extern ID	oj_hash_key_id;
 extern ID	oj_hash_set_id;
 extern ID	oj_hash_start_id;
 extern ID	oj_iconv_id;
@@ -262,9 +267,11 @@ extern ID	oj_instance_variables_id;
 extern ID	oj_json_create_id;
 extern ID	oj_length_id;
 extern ID	oj_new_id;
+extern ID	oj_pos_id;
 extern ID	oj_read_id;
 extern ID	oj_readpartial_id;
 extern ID	oj_replace_id;
+extern ID	oj_stat_id;
 extern ID	oj_string_id;
 extern ID	oj_to_hash_id;
 extern ID	oj_to_json_id;
