@@ -65,16 +65,17 @@ typedef struct _ParseInfo {
 
     struct _Err		err;
     struct _Options	options;
-    void		*cbc;
+    VALUE		handler;
     struct _ValStack	stack;
     CircArray		circ_array;
     int			expect_value;
     VALUE		proc;
     VALUE		(*start_hash)(struct _ParseInfo *pi);
     void		(*end_hash)(struct _ParseInfo *pi);
-    void		(*hash_set_cstr)(struct _ParseInfo *pi, const char *key, size_t klen, const char *str, size_t len, const char *orig);
-    void		(*hash_set_num)(struct _ParseInfo *pi, const char *key, size_t klen, NumInfo ni);
-    void		(*hash_set_value)(struct _ParseInfo *pi, const char *key, size_t klen, VALUE value);
+    VALUE		(*hash_key)(struct _ParseInfo *pi, const char *key, size_t klen);
+    void		(*hash_set_cstr)(struct _ParseInfo *pi, Val kval, const char *str, size_t len, const char *orig);
+    void		(*hash_set_num)(struct _ParseInfo *pi, Val kval, NumInfo ni);
+    void		(*hash_set_value)(struct _ParseInfo *pi, Val kval, VALUE value);
 
     VALUE		(*start_array)(struct _ParseInfo *pi);
     void		(*end_array)(struct _ParseInfo *pi);
@@ -89,6 +90,7 @@ typedef struct _ParseInfo {
 
 extern void	oj_parse2(ParseInfo pi);
 extern void	oj_set_error_at(ParseInfo pi, VALUE err_clas, const char* file, int line, const char *format, ...);
+extern void	oj_pi_set_input_str(ParseInfo pi, volatile VALUE input);
 extern VALUE	oj_pi_parse(int argc, VALUE *argv, ParseInfo pi, char *json, size_t len, int yieldOk);
 extern VALUE	oj_num_as_value(NumInfo ni);
 
