@@ -49,7 +49,7 @@
 #define NUM_MAX		(FIXNUM_MAX >> 8)
 #endif
 #define EXP_MAX		1023
-#define DEC_MAX		14
+#define DEC_MAX		15
 
 static void
 skip_comment(ParseInfo pi) {
@@ -99,7 +99,7 @@ add_value(ParseInfo pi, VALUE rval) {
 	    parent->next = NEXT_ARRAY_COMMA;
 	    break;
 	case NEXT_HASH_VALUE:
-	    pi->hash_set_value(pi, parent->key, parent->klen, rval);
+	    pi->hash_set_value(pi, parent, rval);
 	    if (parent->kalloc) {
 		xfree((char*)parent->key);
 	    }
@@ -134,7 +134,7 @@ add_num_value(ParseInfo pi, NumInfo ni) {
 	    parent->next = NEXT_ARRAY_COMMA;
 	    break;
 	case NEXT_HASH_VALUE:
-	    pi->hash_set_num(pi, parent->key, parent->klen, ni);
+	    pi->hash_set_num(pi, parent, ni);
 	    if (parent->kalloc) {
 		xfree((char*)parent->key);
 	    }
@@ -302,13 +302,18 @@ read_escaped_str(ParseInfo pi) {
 	    break;
 	case NEXT_HASH_NEW:
 	case NEXT_HASH_KEY:
-	    parent->key = strdup(buf.head);
-	    parent->klen = buf_len(&buf);
+	    if (Qundef == (parent->key_val = pi->hash_key(pi, buf.head, buf_len(&buf)))) {
+		parent->key = strdup(buf.head);
+		parent->klen = buf_len(&buf);
+	    } else {
+		parent->key = "";
+		parent->klen = 0;
+	    }
 	    parent->k1 = *pi->rd.str;
 	    parent->next = NEXT_HASH_COLON;
 	    break;
 	case NEXT_HASH_VALUE:
-	    pi->hash_set_cstr(pi, parent->key, parent->klen, buf.head, buf_len(&buf), pi->rd.str);
+	    pi->hash_set_cstr(pi, parent, buf.head, buf_len(&buf), pi->rd.str);
 	    if (parent->kalloc) {
 		xfree((char*)parent->key);
 	    }
@@ -366,11 +371,12 @@ read_str(ParseInfo pi) {
 		parent->key = parent->karray;
 		parent->kalloc = 0;
 	    }
+	    parent->key_val = pi->hash_key(pi, parent->key, parent->klen);
 	    parent->k1 = *pi->rd.str;
 	    parent->next = NEXT_HASH_COLON;
 	    break;
 	case NEXT_HASH_VALUE:
-	    pi->hash_set_cstr(pi, parent->key, parent->klen, pi->rd.str, pi->rd.tail - pi->rd.str - 1, pi->rd.str);
+	    pi->hash_set_cstr(pi, parent, pi->rd.str, pi->rd.tail - pi->rd.str - 1, pi->rd.str);
 	    if (parent->kalloc) {
 		xfree((char*)parent->key);
 	    }
@@ -739,8 +745,6 @@ oj_pi_sparse(int argc, VALUE *argv, ParseInfo pi, int fd) {
     } else {
 	pi->proc = Qundef;
     }
-    pi->cbc = (void*)0;
-
     oj_reader_init(&pi->rd, input, fd);
     pi->json = 0; // indicates reader is in use
 
