@@ -1,84 +1,70 @@
 #!/usr/bin/env ruby
 # encoding: UTF-8
 
-# Ubuntu does not accept arguments to ruby when called using env. To get warnings to show up the -w options is
-# required. That can be set in the RUBYOPT environment variable.
-# export RUBYOPT=-w
+$: << File.dirname(__FILE__)
 
-$VERBOSE = true
+require 'helper'
 
-$: << File.join(File.dirname(__FILE__), "../lib")
-$: << File.join(File.dirname(__FILE__), "../ext")
+class CompatJuice < Minitest::Test
 
-require 'test/unit'
-require 'stringio'
-require 'date'
-require 'bigdecimal'
-require 'oj'
+  class Jeez
+    attr_accessor :x, :y
 
-$ruby = RUBY_DESCRIPTION.split(' ')[0]
-$ruby = 'ree' if 'ruby' == $ruby && RUBY_DESCRIPTION.include?('Ruby Enterprise Edition')
+    def initialize(x, y)
+      @x = x
+      @y = y
+    end
 
-class Jeez
-  attr_accessor :x, :y
+    def eql?(o)
+      self.class == o.class && @x == o.x && @y == o.y
+    end
+    alias == eql?
 
-  def initialize(x, y)
-    @x = x
-    @y = y
+    def as_json()
+      {"json_class" => self.class.to_s,"x" => @x,"y" => @y}
+    end
+
+    def self.json_create(h)
+      self.new(h['x'], h['y'])
+    end
+  end # Jeez
+
+  module One
+    module Two
+      module Three
+        class Deep
+
+          def initialize()
+          end
+
+          def eql?(o)
+            self.class == o.class
+          end
+          alias == eql?
+
+          def to_hash()
+            {'json_class' => "#{self.class.name}"}
+          end
+
+          def to_json(*a)
+            %{{"json_class":"#{self.class.name}"}}
+          end
+
+          def self.json_create(h)
+            self.new()
+          end
+        end # Deep
+      end # Three
+    end # Two
+  end # One
+
+  def setup
+    @default_options = Oj.default_options
   end
 
-  def eql?(o)
-    self.class == o.class && @x == o.x && @y == o.y
+  def teardown
+    Oj.default_options = @default_options
   end
-  alias == eql?
-  
-  def as_json()
-    {"json_class" => self.class.to_s,"x" => @x,"y" => @y}
-  end
-
-  def self.json_create(h)
-    self.new(h['x'], h['y'])
-  end
-end # Jeez
-
-module One
-  module Two
-    module Three
-      class Deep
-
-        def initialize()
-        end
-
-        def eql?(o)
-          self.class == o.class
-        end
-        alias == eql?
-
-        def to_hash()
-          {'json_class' => "#{self.class.name}"}
-        end
-
-        def to_json(*a)
-          %{{"json_class":"#{self.class.name}"}}
-        end
-
-        def self.json_create(h)
-          self.new()
-        end
-      end # Deep
-    end # Three
-  end # Two
-end # One
-
-def hash_eql(h1, h2)
-  return false if h1.size != h2.size
-  h1.keys.each do |k|
-    return false unless h1[k] == h2[k]
-  end
-  true
-end
-
-class CompatJuice < ::Test::Unit::TestCase
 
   def test_nil
     dump_and_load(nil, false)
@@ -104,6 +90,8 @@ class CompatJuice < ::Test::Unit::TestCase
     dump_and_load(12345.6789, false)
     dump_and_load(70.35, false)
     dump_and_load(-54321.012, false)
+    dump_and_load(1.7775, false)
+    dump_and_load(2.5024, false)
     dump_and_load(2.48e16, false)
     dump_and_load(2.48e100 * 1.0e10, false)
     dump_and_load(-2.48e100 * 1.0e10, false)
@@ -218,7 +206,7 @@ class CompatJuice < ::Test::Unit::TestCase
   end
 
   def test_io_file
-    filename = 'open_file_test.json'
+    filename = File.join(File.dirname(__FILE__), 'open_file_test.json')
     File.open(filename, 'w') { |f| f.write(%{{
   "x":true,
   "y":58,
@@ -281,6 +269,7 @@ class CompatJuice < ::Test::Unit::TestCase
 
   def test_json_object_compat
     obj = Jeez.new(true, 58)
+    Oj.default_options = { :mode => :compat, :use_to_json => true }
     dump_and_load(obj, false)
   end
 
@@ -291,13 +280,13 @@ class CompatJuice < ::Test::Unit::TestCase
 
   def test_json_object_create_id
     expected = Jeez.new(true, 58)
-    json = Oj.dump(expected, :indent => 2, :mode => :compat)
+    json = Oj.dump(expected, :indent => 2, :mode => :compat, :use_to_json => true)
     obj = Oj.compat_load(json)
     assert_equal(expected, obj)
   end
 
   def test_json_object_bad
-    json = %{{"json_class":"Junk","x":true}}
+    json = %{{"json_class":"CompatJuice::Junk","x":true}}
     begin
       Oj.compat_load(json)
     rescue Exception => e
@@ -309,7 +298,7 @@ class CompatJuice < ::Test::Unit::TestCase
 
   def test_json_object_create_cache
     expected = Jeez.new(true, 58)
-    json = Oj.dump(expected, :indent => 2, :mode => :compat)
+    json = Oj.dump(expected, :indent => 2, :mode => :compat, :use_to_json => true)
     obj = Oj.compat_load(json, :class_cache => true)
     assert_equal(expected, obj)
     obj = Oj.compat_load(json, :class_cache => false)
@@ -318,7 +307,7 @@ class CompatJuice < ::Test::Unit::TestCase
 
   def test_json_object_create_id_other
     expected = Jeez.new(true, 58)
-    json = Oj.dump(expected, :indent => 2, :mode => :compat)
+    json = Oj.dump(expected, :indent => 2, :mode => :compat, :use_to_json => true)
     json.gsub!('json_class', '_class_')
     obj = Oj.compat_load(json, :create_id => "_class_")
     assert_equal(expected, obj)
