@@ -1,158 +1,143 @@
 #!/usr/bin/env ruby
 # encoding: UTF-8
 
-# Ubuntu does not accept arguments to ruby when called using env. To get warnings to show up the -w options is
-# required. That can be set in the RUBYOPT environment variable.
-# export RUBYOPT=-w
+$: << File.dirname(__FILE__)
 
-$VERBOSE = true
+require 'helper'
 
-$: << File.join(File.dirname(__FILE__), "../lib")
-$: << File.join(File.dirname(__FILE__), "../ext")
+class ObjectJuice < Minitest::Test
+  class Jeez
+    attr_accessor :x, :y
 
-require 'test/unit'
-require 'stringio'
-require 'date'
-require 'bigdecimal'
-require 'oj'
+    def initialize(x, y)
+      @x = x
+      @y = y
+    end
 
-$ruby = RUBY_DESCRIPTION.split(' ')[0]
-$ruby = 'ree' if 'ruby' == $ruby && RUBY_DESCRIPTION.include?('Ruby Enterprise Edition')
+    def eql?(o)
+      self.class == o.class && @x == o.x && @y == o.y
+    end
+    alias == eql?
 
-class Jeez
-  attr_accessor :x, :y
+    def to_json(*a)
+      %{{"json_class":"#{self.class}","x":#{@x},"y":#{@y}}}
+    end
 
-  def initialize(x, y)
-    @x = x
-    @y = y
+    def self.json_create(h)
+      self.new(h['x'], h['y'])
+    end
+  end # Jeez
+
+  module One
+    module Two
+      module Three
+        class Deep
+
+          def initialize()
+          end
+
+          def eql?(o)
+            self.class == o.class
+          end
+          alias == eql?
+
+          def to_hash()
+            {'json_class' => "#{self.class.name}"}
+          end
+
+          def to_json(*a)
+            %{{"json_class":"#{self.class.name}"}}
+          end
+
+          def self.json_create(h)
+            self.new()
+          end
+        end # Deep
+      end # Three
+    end # Two
+
+    class Stuck2 < Struct.new(:a, :b)
+    end
+
+  end # One
+
+  class Stuck < Struct.new(:a, :b)
   end
 
-  def eql?(o)
-    self.class == o.class && @x == o.x && @y == o.y
-  end
-  alias == eql?
-  
-  def to_json(*a)
-    %{{"json_class":"#{self.class}","x":#{@x},"y":#{@y}}}
+  class Strung < String
+
+    def initialize(str, safe)
+      super(str)
+      @safe = safe
+    end
+
+    def safe?()
+      @safe
+    end
+
+    def self.create(str, safe)
+      new(str, safe)
+    end
+
+    def eql?(o)
+      super && self.class == o.class && @safe == o.safe?
+    end
+    alias == eql?
+
+    def inspect()
+      return super + '(' + @safe + ')'
+    end
   end
 
-  def self.json_create(h)
-    self.new(h['x'], h['y'])
-  end
-end # Jeez
+  class AutoStrung < String
+    attr_accessor :safe
 
-module One
-  module Two
-    module Three
-      class Deep
+    def initialize(str, safe)
+      super(str)
+      @safe = safe
+    end
 
-        def initialize()
-        end
-
-        def eql?(o)
-          self.class == o.class
-        end
-        alias == eql?
-
-        def to_hash()
-          {'json_class' => "#{self.class.name}"}
-        end
-
-        def to_json(*a)
-          %{{"json_class":"#{self.class.name}"}}
-        end
-
-        def self.json_create(h)
-          self.new()
-        end
-      end # Deep
-    end # Three
-  end # Two
-
-  class Stuck2 < Struct.new(:a, :b)
+    def eql?(o)
+      self.class == o.class && super(o) && @safe == o.safe
+    end
+    alias == eql?
   end
 
-end # One
+  class AutoArray < Array
+    attr_accessor :safe
 
-class Stuck < Struct.new(:a, :b)
-end
+    def initialize(a, safe)
+      super(a)
+      @safe = safe
+    end
 
-class Strung < String
-
-  def initialize(str, safe)
-    super(str)
-    @safe = safe
+    def eql?(o)
+      self.class == o.class && super(o) && @safe == o.safe
+    end
+    alias == eql?
   end
 
-  def safe?()
-    @safe
+  class AutoHash < Hash
+    attr_accessor :safe
+
+    def initialize(h, safe)
+      super(h)
+      @safe = safe
+    end
+
+    def eql?(o)
+      self.class == o.class && super(o) && @safe == o.safe
+    end
+    alias == eql?
   end
 
-  def self.create(str, safe)
-    new(str, safe)
+  def setup
+    @default_options = Oj.default_options
   end
 
-  def eql?(o)
-    super && self.class == o.class && @safe == o.safe?
+  def teardown
+    Oj.default_options = @default_options
   end
-  alias == eql?
-
-  def inspect()
-    return super + '(' + @safe + ')'
-  end
-end
-
-class AutoStrung < String
-  attr_accessor :safe
-
-  def initialize(str, safe)
-    super(str)
-    @safe = safe
-  end
-
-  def eql?(o)
-    self.class == o.class && super(o) && @safe == o.safe
-  end
-  alias == eql?
-end
-
-class AutoArray < Array
-  attr_accessor :safe
-
-  def initialize(a, safe)
-    super(a)
-    @safe = safe
-  end
-
-  def eql?(o)
-    self.class == o.class && super(o) && @safe == o.safe
-  end
-  alias == eql?
-end
-
-class AutoHash < Hash
-  attr_accessor :safe
-
-  def initialize(h, safe)
-    super(h)
-    @safe = safe
-  end
-
-  def eql?(o)
-    self.class == o.class && super(o) && @safe == o.safe
-  end
-  alias == eql?
-end
-
-def hash_eql(h1, h2)
-  return false if h1.size != h2.size
-  h1.keys.each do |k|
-    return false unless h1[k] == h2[k]
-  end
-  true
-end
-
-class ObjectJuice < ::Test::Unit::TestCase
 
   def test_nil
     dump_and_load(nil, false)
@@ -178,6 +163,8 @@ class ObjectJuice < ::Test::Unit::TestCase
     dump_and_load(12345.6789, false)
     dump_and_load(70.35, false)
     dump_and_load(-54321.012, false)
+    dump_and_load(1.7775, false)
+    dump_and_load(2.5024, false)
     dump_and_load(2.48e16, false)
     dump_and_load(2.48e100 * 1.0e10, false)
     dump_and_load(-2.48e100 * 1.0e10, false)
@@ -297,7 +284,7 @@ class ObjectJuice < ::Test::Unit::TestCase
   end
 
   def test_io_file
-    filename = 'open_file_test.json'
+    filename = File.join(File.dirname(__FILE__), 'open_file_test.json')
     File.open(filename, 'w') { |f| f.write(%{{
   "x":true,
   "y":58,
@@ -368,6 +355,24 @@ class ObjectJuice < ::Test::Unit::TestCase
     dump_and_load(t, false)
   end
 
+  def test_xml_time
+    Oj.default_options = { :mode => :object, :time_format => :xmlschema }
+    t = Time.now()
+    dump_and_load(t, false)
+  end
+
+  def test_utc_time
+    Oj.default_options = { :mode => :object, :time_format => :xmlschema }
+    t = Time.now().utc
+    dump_and_load(t, false)
+  end
+
+  def test_ruby_time
+    Oj.default_options = { :mode => :object, :time_format => :ruby }
+    t = Time.now()
+    dump_and_load(t, false)
+  end
+
   def test_time_early
     t = Time.xmlschema("1954-01-05T00:00:00.123456")
     dump_and_load(t, false)
@@ -394,6 +399,16 @@ class ObjectJuice < ::Test::Unit::TestCase
     assert(false, "*** expected an exception")
   end
 
+  def test_json_object_not_hat_hash
+    json = %{{"^#x":[1,2]}}
+    h = Oj.object_load(json)
+    assert_equal({1 => 2}, h);
+
+    json = %{{"~#x":[1,2]}}
+    h = Oj.object_load(json)
+    assert_equal({'~#x' => [1,2]}, h);
+  end
+
   def test_json_struct
     unless 'jruby' == RUBY_DESCRIPTION.split(' ')[0]
       obj = Stuck.new(false, 7)
@@ -414,6 +429,7 @@ class ObjectJuice < ::Test::Unit::TestCase
   end
 
   def test_mixed_hash_object
+    Oj.default_options = { :mode => :object }
     json = Oj.dump({ 1 => true, 'nil' => nil, :sim => 4 })
     h = Oj.object_load(json)
     assert_equal({ 1 => true, 'nil' => nil, :sim => 4 }, h)
