@@ -1,18 +1,9 @@
 #!/usr/bin/env ruby
 # encoding: UTF-8
 
-# Ubuntu does not accept arguments to ruby when called using env. To get warnings to show up the -w options is
-# required. That can be set in the RUBYOPT environment variable.
-# export RUBYOPT=-w
+$: << File.dirname(__FILE__)
 
-$VERBOSE = true
-
-$: << File.join(File.dirname(__FILE__), "../lib")
-$: << File.join(File.dirname(__FILE__), "../ext")
-
-require 'test/unit'
-require 'oj'
-require 'pp'
+require 'helper'
 
 $json = %{{
   "array": [
@@ -50,6 +41,13 @@ class AllHandler < Oj::ScHandler
     @calls << [:hash_end]
   end
 
+  def hash_key(key)
+    @calls << [:hash_key, key]
+    return 'too' if 'two' == key
+    return :symbol if 'symbol' == key
+    key
+  end
+
   def array_start()
     @calls << [:array_start]
     []
@@ -66,14 +64,22 @@ class AllHandler < Oj::ScHandler
   def hash_set(h, key, value)
     @calls << [:hash_set, key, value]
   end
-  
+
   def array_append(a, value)
     @calls << [:array_append, value]
   end
 
 end # AllHandler
 
-class ScpTest < ::Test::Unit::TestCase
+class ScpTest < Minitest::Test
+
+  def setup
+    @default_options = Oj.default_options
+  end
+
+  def teardown
+    Oj.default_options = @default_options
+  end
 
   def test_nil
     handler = AllHandler.new()
@@ -160,8 +166,10 @@ class ScpTest < ::Test::Unit::TestCase
     json = %{{"one":true,"two":false}}
     Oj.sc_parse(handler, json)
     assert_equal([[:hash_start],
+                  [:hash_key, 'one'],
                   [:hash_set, 'one', true],
-                  [:hash_set, 'two', false],
+                  [:hash_key, 'two'],
+                  [:hash_set, 'too', false],
                   [:hash_end],
                   [:add_value, {}]], handler.calls)
   end
@@ -171,8 +179,23 @@ class ScpTest < ::Test::Unit::TestCase
     json = %{{"one":true,"two":false}}
     Oj.sc_parse(handler, json, :symbol_keys => true)
     assert_equal([[:hash_start],
-                  [:hash_set, :one, true],
-                  [:hash_set, :two, false],
+                  [:hash_key, 'one'],
+                  [:hash_set, 'one', true],
+                  [:hash_key, 'two'],
+                  [:hash_set, 'too', false],
+                  [:hash_end],
+                  [:add_value, {}]], handler.calls)
+  end
+
+  def test_symbol_hash_key_without_symbol_keys
+    handler = AllHandler.new()
+    json = %{{"one":true,"symbol":false}}
+    Oj.sc_parse(handler, json)
+    assert_equal([[:hash_start],
+                  [:hash_key, 'one'],
+                  [:hash_set, 'one', true],
+                  [:hash_key, 'symbol'],
+                  [:hash_set, :symbol, false],
                   [:hash_end],
                   [:add_value, {}]], handler.calls)
   end
@@ -181,12 +204,18 @@ class ScpTest < ::Test::Unit::TestCase
     handler = AllHandler.new()
     Oj.sc_parse(handler, $json)
     assert_equal([[:hash_start],
+                  [:hash_key, 'array'],
                   [:array_start],
                   [:hash_start],
+                  [:hash_key, 'num'],
                   [:hash_set, "num", 3],
+                  [:hash_key, 'string'],
                   [:hash_set, "string", "message"],
+                  [:hash_key, 'hash'],
                   [:hash_start],
+                  [:hash_key, 'h2'],
                   [:hash_start],
+                  [:hash_key, 'a'],
                   [:array_start],
                   [:array_append, 1],
                   [:array_append, 2],
@@ -201,6 +230,7 @@ class ScpTest < ::Test::Unit::TestCase
                   [:array_append, {}],
                   [:array_end],
                   [:hash_set, "array", []],
+                  [:hash_key, 'boolean'],
                   [:hash_set, "boolean", true],
                   [:hash_end],
                   [:add_value, {}]], handler.calls)
@@ -211,12 +241,16 @@ class ScpTest < ::Test::Unit::TestCase
     json = %{{"one":true,"two":false}{"three":true,"four":false}}
     Oj.sc_parse(handler, json)
     assert_equal([[:hash_start],
+                  [:hash_key, 'one'],
                   [:hash_set, 'one', true],
-                  [:hash_set, 'two', false],
+                  [:hash_key, 'two'],
+                  [:hash_set, 'too', false],
                   [:hash_end],
                   [:add_value, {}],
                   [:hash_start],
+                  [:hash_key, 'three'],
                   [:hash_set, 'three', true],
+                  [:hash_key, 'four'],
                   [:hash_set, 'four', false],
                   [:hash_end],
                   [:add_value, {}]], handler.calls)
@@ -233,7 +267,39 @@ class ScpTest < ::Test::Unit::TestCase
     begin
       Oj.sc_parse(handler, json)
     rescue Exception => e
-      assert_equal("unexpected character at line 1, column 6 [parse.c:637]", e.message)
+      assert_match(%r{unexpected character at line 1, column 6 \[(?:[a-z\.]+/)*parse\.c:\d+\]}, e.message)
+    end
+  end
+
+  def test_null_string
+    handler = AllHandler.new()
+    json = %{"\0"}
+    assert_raises Oj::ParseError do
+      Oj.sc_parse(handler, json)
+    end
+  end
+
+  def test_pipe
+    handler = AllHandler.new()
+    json = %{{"one":true,"two":false}}
+    IO.pipe do |read_io, write_io|
+      if fork
+        write_io.close
+        Oj.sc_parse(handler, read_io) {|v| p v}
+        read_io.close
+        assert_equal([[:hash_start],
+                      [:hash_key, 'one'],
+                      [:hash_set, 'one', true],
+                      [:hash_key, 'two'],
+                      [:hash_set, 'too', false],
+                      [:hash_end],
+                      [:add_value, {}]], handler.calls)
+      else
+        read_io.close
+        write_io.write json
+        write_io.close
+        Process.exit(0)
+      end
     end
   end
 
